﻿using System;
using System.IO;
using System.Threading;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.DynamicAssemblyTestApp {
  [Serializable]
  [Application("CLIOptimize", "")]
  public class AppTest : ApplicationBase {

    #region Vars
    [NonSerialized]
    private IExecutable executable;
    [NonSerialized]
    private static AutoResetEvent autoResetEvent = new AutoResetEvent(false);
    [NonSerialized]
    private static object locker = new object();
    #endregion

    #region Properties
    public UniPath InputFilePath { get; set; } = null;
    public UniPath OutputPath { get; set; } = null;
    #endregion

    #region Constructors
    public AppTest() { }
    public AppTest(UniPath inputFilePath, UniPath outputPath) {
      InputFilePath = inputFilePath;
      OutputPath = outputPath;
    }
    #endregion

    public override void Run(ICommandLineArgument[] args) {
      Init();
      lock (locker) {
        executable.Stopped += Executable_Stopped;
        executable.StartAsync();
      }
      autoResetEvent.WaitOne();
    }

    public override void OnCancel() {
      lock (locker) {
        base.OnCancel();
        if (executable != null)
          executable.Stop();
      }
    }

    public override void OnPause() {
      base.OnPause();
      lock (locker) {
        if (executable != null) executable.Pause();
      }
    }

    public override void OnResume() {
      base.OnResume();
      lock (locker) {
        if (executable != null)
          executable.StartAsync();
      }
    }

    #region Helper
    private void Init() {
      lock (locker) {
        ContentManager.Initialize(new PersistenceContentManager());
        IStorableContent content = ContentManager.Load(InputFilePath.ToString());
        executable = content as IExecutable;
        if (executable == null)
          throw new NotSupportedException("The given file does not contain any algorithm to start.");
      }
    }

    private void Executable_Stopped(object sender, EventArgs e) {
      lock (locker) {
        IOptimizer optimizer = executable as IOptimizer;
        if (optimizer != null) PrintResults(optimizer);
        ContentManager.Save((IStorableContent)executable, OutputPath.ToString() + Path.DirectorySeparatorChar + "result.hl", true);
      }
      autoResetEvent.Set();
    }

    private void PrintResults(IOptimizer optimizer) {
      lock (locker) {
        Console.WriteLine("\nRESULT(S):");
        int i = 1;
        foreach (var run in optimizer.Runs) {
          Console.WriteLine($"{"-------------------------------- RUN",35} {$"{i++:D3}" + " --------------------------------",-35}");
          foreach (var res in run.Results) {
            Console.WriteLine($"{res.Key,35} : {res.Value,-35}");
          }
        }
      }
    }
    #endregion
  }
}
